#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
import logging
import os
import subprocess
import sys

from casa_envoy.interfaces import RunnerIF


class CasaConfigure:
    """
    Class for unifying environment setup. Required by all CASA Runner types
    """

    def __init__(self, parameters: dict, content: dict):
        self.logger = logging.getLogger("casa_envoy")
        self.parameters = parameters
        self.metadata_content = content

    def setup_environment(self):
        """
        Set required environment variables

        :return:
        """
        os.environ["SCIPIPE_ROOTDIR"] = self.parameters["rootDirectory"]

        # If user specified CASA Version, use it
        if "casaHome" in self.metadata_content:
            os.environ["CASA_HOME"] = self.metadata_content.get("casaHome")
        else:
            # If VLASS, use the current VLASS CASA release
            if "vlass" in self.parameters["product_type"]:
                os.environ["CASA_HOME"] = self.parameters["vlassHome"]
            else:
                os.environ["CASA_HOME"] = self.parameters["homeForReprocessing"]

        os.environ["LANG"] = "en_US.UTF-8"
        os.environ["PPR_FILENAME"] = self.parameters["ppr"]

        self.check_processing_env()

    def check_processing_env(self):
        """
        Varify processing environment is ready for CASA

        :return:
        """
        self.logger.info("Checking processing environment:")

        env_list = ["SCIPIPE_ROOTDIR", "CASA_HOME", "PPR_FILENAME", "LANG"]
        result_list = []

        for var in env_list:
            env = os.environ.get(var)
            result_list.append(env)
            self.logger.info(f"{var}: {env}")

        if "None" in result_list:
            self.logger.error("Environment setup Failed!")
            sys.exit(1)
        else:
            self.logger.info("Environment ready for processing")


class StandardRunner(RunnerIF):
    """
    Class for launch of standard (non-parallel) CASA
    """

    def __init__(self, parameters: dict):
        self.logger = logging.getLogger("casa_envoy")
        self.parameters = parameters

    def call_casa(self):
        """
        Launch subprocess for standard CASA

        :return:
        """
        build_command = (
            "PYTHONPATH='' xvfb-run -e ${PWD}/xvfb-run.err.txt -d -s \"-screen 0 800x600x16\" "
            "${CASA_HOME}/bin/casa --pipeline --nogui --nologger -c "
            "${CASA_HOME}/pipeline/pipeline/runvlapipeline.py ${PPR_FILENAME} || true"
        )

        self.logger.info(f"RUNNING CASA with command '{build_command}'")
        os.chdir("./working")
        result = subprocess.Popen(
            build_command,
            shell=True,
            executable="/bin/bash",
            stdout=sys.stdout,
            stderr=sys.stderr,
        )
        return result.communicate()


class ParallelRunner(RunnerIF):
    """
    Class for launch of parallel CASA processing using mpicasa
    """

    def __init__(self, parameters: dict):
        self.logger = logging.getLogger("casa_envoy")
        self.parameters = parameters

    def call_casa(self):

        build_command = (
            "PYTHONPATH='' xvfb-run -e ${PWD}/xvfb-run.err.txt -d -s \"-screen 0 800x600x16\""
            " ${CASA_HOME}/bin/mpicasa -n "
            + self.parameters["requested_parallel"]
            + "${CASA_HOME}/bin/casa --pipeline --nogui --nologger -c "
            "${CASA_HOME}/pipeline/pipeline/runvlapipeline.py ${PPR_FILENAME} || true"
        )

        self.logger.info(f"RUNNING PARALLEL CASA with command '{build_command}'")
        os.chdir("./working")
        result = subprocess.Popen(
            build_command,
            shell=True,
            executable="/bin/bash",
            stdout=sys.stdout,
            stderr=sys.stderr,
        )
        return result.communicate()
